/* SPDX-License-Identifier: GPL-2.0 */
#ifndef __IP_SET_LIST_H
#define __IP_SET_LIST_H

#include <uapi/linux/netfilter/ipset/ip_set_list.h>


#define IP_SET_LIST_DEFAULT_SIZE	8
#define IP_SET_LIST_MIN_SIZE		4
#define IP_SET_LIST_MAX_SIZE		65536

#endif /* __IP_SET_LIST_H */
