/* SPDX-License-Identifier: GPL-2.0 */
#ifndef _DT_BINDINGS_POWER_MT7623A_POWER_H
#define _DT_BINDINGS_POWER_MT7623A_POWER_H

#define MT7623A_POWER_DOMAIN_CONN	0
#define MT7623A_POWER_DOMAIN_ETH	1
#define MT7623A_POWER_DOMAIN_HIF	2
#define MT7623A_POWER_DOMAIN_IFR_MSC	3

#endif /* _DT_BINDINGS_POWER_MT7623A_POWER_H */
