/* SPDX-License-Identifier: GPL-2.0-only */
/*
 *  Copyright (C) 2017 Impinj
 */

#ifndef __DT_BINDINGS_IMX7_POWER_H__
#define __DT_BINDINGS_IMX7_POWER_H__

#define IMX7_POWER_DOMAIN_MIPI_PHY		0
#define IMX7_POWER_DOMAIN_PCIE_PHY		1
#define IMX7_POWER_DOMAIN_USB_HSIC_PHY		2

#endif
