/* SPDX-License-Identifier: GPL-2.0 */
/*
 * SGI One-Wire (W1) IP
 */

#ifndef PLATFORM_DATA_SGI_W1_H
#define PLATFORM_DATA_SGI_W1_H

struct sgi_w1_platform_data {
	char dev_id[64];
};

#endif /* PLATFORM_DATA_SGI_W1_H */
